# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import unicodedata
import six

from driverng import MessageV2, Worklist
from driverng.splitted import Multiple, Splitted, String


class AgilentResultsBase(Splitted):

    SEPARATOR = ","


class AgilentResult(object):
    FIELDS = [
        String("well"),
        String("well_type"),
        String("sample_name"),
        String("well_name"),
        String("dye"),
        String("target"),
        Multiple("results", (String("resu"),), separator=","),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for i, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[i:])
                break
            else:
                setattr(self, field.name, line[i])


class AgilentResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: AgilentResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf_data = unicodedata.normalize(
            "NFKD", six.text_type(data.replace("\u2206", "D"))
        ).encode("ascii", "ignore")
        if six.PY3:
            buf_data = buf_data.decode("ascii")
        buf = six.StringIO(buf_data)
        reader = csv.reader(buf, delimiter=str(","))
        for line in reader:
            if not len(line):
                continue
            if len(line) > 10:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "agilent", "label": "AGILENT ARIA"}

    def parse(self, data):
        obj = AgilentResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if m.well.strip() == "Well" and m.well_type.strip() == "Well Type":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample_name.strip(), rack=m.well, type="patient"
                )
                for index, result in enumerate(m.results):
                    if result == "N/A":
                        continue
                    if RESULTS_CODES[index] and result:
                        sample.results.add(
                            code="{}{}".format(m.dye, RESULTS_CODES[index].strip()),
                            value=result.strip(),
                        )
        return worklist
