# -*- coding: utf8 -*-
from __future__ import unicode_literals

import logging

import xml.etree.cElementTree as ET
from driverng import MessageV2, utils

logger = logging.getLogger("driverng.e4l_facturation_xml")


class Driver(MessageV2):
    SPEC = {"code": "e4l_facturation_xml", "label": "Facturation E4L (XML)"}

    def parse(self, data):
        return ET.fromstring(data.encode("utf8"))

    def to_string(self, message):
        return utils.prettify_xml(ET.tostring(message, "utf-8"))

    def encode_worklist(self, worklist, context):
        for sample in worklist.samples:
            if sample.billings:
                for billing in sample.billings:
                    specification_name = billing.specification_name
                    specification_version = billing.specification_version
                    code_executant = billing.code_executant
                    nom_executant = billing.nom_executant
                    total_amount = billing.total_amount
                    unpaid_amount = (
                        billing.unpaid_amount if billing.unpaid_amount else "0"
                    )
                    currency = billing.currency
                    fee_note = billing.fee_note
                    receipt = billing.receipt
                message = ET.Element("financial")
                fileinfo = ET.SubElement(message, "fileinfo")
                ET.SubElement(fileinfo, "filename").text = worklist.filename
                ET.SubElement(fileinfo, "datetime").text = utils.format_date(
                    context["now"], fmt="%Y%m%d%H%M%S"
                )
                specification = ET.SubElement(fileinfo, "specification")
                ET.SubElement(
                    specification, "specificationName"
                ).text = specification_name
                ET.SubElement(
                    specification, "specificationVersion"
                ).text = specification_version
                organization = ET.SubElement(message, "organization")
                ET.SubElement(organization, "idSite").text = worklist.sender_id
                ET.SubElement(organization, "nameSite").text = worklist.sender_name
                ET.SubElement(organization, "idLab").text = code_executant
                ET.SubElement(organization, "nameLab").text = nom_executant
                patient = ET.SubElement(message, "patient")
                ET.SubElement(patient, "localId").text = sample.patient_id
                ins = ET.SubElement(patient, "INS")
                ET.SubElement(ins, "value").text = (
                    sample.ins_number if sample.identity_status == "Qualifiée" else ""
                )
                ET.SubElement(ins, "OID").text = (
                    sample.oid if sample.identity_status == "Qualifiée" else ""
                )
                ET.SubElement(patient, "familyName").text = sample.last_name
                ET.SubElement(patient, "firstName").text = sample.first_name
                further_names = ""
                if sample.further_names:
                    further_names = " ".join(sample.further_names)
                ET.SubElement(patient, "firstNames").text = further_names
                ET.SubElement(patient, "sex").text = sample.sex
                ET.SubElement(patient, "birthDay").text = utils.format_date(
                    sample.birthdate, fmt="%Y%m%d"
                )
                ET.SubElement(patient, "birthPlace").text = sample.birthplace_code
                request = ET.SubElement(message, "request")
                ET.SubElement(request, "id").text = sample.request_id
                ET.SubElement(request, "totalAmount").text = total_amount
                ET.SubElement(request, "unpaidAmount").text = (
                    unpaid_amount if int(unpaid_amount) >= 0 else "0"
                )
                ET.SubElement(request, "currency").text = currency
                attached_files = ET.SubElement(request, "attachedFiles")
                fee_note_file = ET.SubElement(attached_files, "feeNote")
                fee_note_file.set("mime", "application/pdf")
                fee_content = ET.SubElement(fee_note_file, "content")
                fee_content.text = fee_note
                receip_file = ET.SubElement(attached_files, "receipt")
                receip_file.set("mime", "application/pdf")
                receipt_content = ET.SubElement(receip_file, "content")
                receipt_content.text = receipt

                return message
            else:
                logger.error(
                    "Pas de facturation pour la demande numéro {}".format(
                        sample.request_id
                    )
                )
