# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "atellica800_cytologie"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].orders[0].dilution = "10"
    message = self.order(
        worklist=worklist,
        context={"control_id": "1521651964637", "awos_id": "1521651964639"},
    )
    expected = """
    MSH|^~\\&|||||20130715141912||OML^O33^OML_O33|1521651964637|P|2.5|||NE|AL||ASCII
    PID|||036666666||NYME ANO
    SPM|1|123456||UR^Urine^HL70487|||||||P^^
    ORC|NW
    OBR|1|123456
    SPM|1|123456||UR^Urine^HL70487|||||||P^^
    ORC|NW
    OBR|1|123456
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    MSH|^~\\&|Atellica UAS800^1||||20180727154737||QBP^Q11^QBP_Q11|20180727154737508|P|2.5|||NE|AL||ASCII
    QPD|WOS^Work Order Step|IHELAW||01416
    RCP|I|RD
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "01416"}],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2017, 10, 30, 16, 25, 52)
    worklist = self.decode(
        """
    MSH|^~\\&|Atellica UAS800^1||||20171030162921||OUL^R22^OUL_R22|20171030162921434|P|2.5|||NE|AL||ASCII
    SFT|Atellica UAS 800|4.0.90.5575|Atellica UAS 800|4.0.90.5575
    PID|1||1||QC_LOW
    SPM|1|0296||UR^Urine^HL70487|||||||Q^Control specimen^HL70369
    SAC||||||||||1|5
    OBR|1|||UrineSedimentResult
    ORC|RE||0296||CM|||||test|Automatically validated
    NTE|1||Test Passed!
    NTE|2||Lot ID: 1111|RE
    NTE|3||Liquid Type: Quantimetrix QuanTscopics|RE
    NTE|4||Expiration Date: 20180904|RE
    OBX|1|ST|798-9^RBC^LN|1|<4|p/ul||N|||F|||||||H100016|20171030162552
    OBX|3|ST|51487-7^WBC^LN|2|<3|p/ul||N|||F|||||||H100016|20171030162552
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "RBC",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "p/ul",
                        "value": "<4",
                    },
                    {
                        "code": "WBC",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "p/ul",
                        "value": "<3",
                    },
                ],
                "control_lot": "1111",
                "sample_id": "0296",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_chemical(self):
    date = datetime.datetime(2017, 10, 27, 9, 8, 1)
    worklist = self.decode(
        """
    MSH|^~\\&|CLINITEKNovus||||20171027094314||OUL^R22^OUL_R22|20171027094314829|P|2.5||||||ASCII
    SFT|CLINITEK Novus|1.2.0.9000000|CLINITEK Novus|1.2.0.9000000
    PID|1
    SPM|1|0064||UR|||||||P
    SAC||||||||||1|5
    INV|CLINITEK Novus 10|OK||||||||||20171103015522||||K372078
    OBR|1|0-08782||CLINITEK Novus 10|||||||||||||||||||||F
    NTE|1||Chemical comment in user sw
    NTE|2||ResultComment:COMMENT 1:NOVUS COMM1|RE
    NTE|3||ResultComment:COMMENT 2:N COMM2|RE
    NTE|4||ResultComment:COMMENT 3:N COMM3|RE
    NTE|5||ResultComment:COMMENT 4:N COMM4|RE
    OBX|1|ST|BIL|1|Negative|||N|||F|||||user1||S001802|20171027090801
    TCD|BIL
    NTE|1||Note for BIL
    OBX|2|ST|BLO|2|Small|||A|||F|||||user1||S001802|20171027090801
    TCD|BLO
    OBX|3|ST|CLA|3|Cloudy|||A|||F|||||user1||S001802|20171027090801
    TCD|CLA
    OBX|4|ST|COL|4|Red|||A|||F|||||user1||S001802|20171027090801
    TCD|COL
    OBX|5|ST|GLU|5|Negative|||N|||F|||||user1||S001802|20171027090801
    TCD|GLU
    OBX|6|ST|KET|6|Negative|||N|||F|||||user1||S001802|20171027090801
    TCD|KET
    NTE|1||Note for KET
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "BILQUAL",
                        "date": date,
                        "flags": [],
                        "instrument": "S001802",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 11, 3, 1, 55, 22),
                                "lot": "K372078",
                                "type": "reagent",
                            }
                        ],
                        "operator": "user1",
                        "unit": "",
                        "value": "Negative",
                    },
                    {
                        "code": "BLOQUAL",
                        "date": date,
                        "flags": ["A"],
                        "instrument": "S001802",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 11, 3, 1, 55, 22),
                                "lot": "K372078",
                                "type": "reagent",
                            }
                        ],
                        "operator": "user1",
                        "unit": "",
                        "value": "Small",
                    },
                    {
                        "code": "CLAQUAL",
                        "date": date,
                        "flags": ["A"],
                        "instrument": "S001802",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 11, 3, 1, 55, 22),
                                "lot": "K372078",
                                "type": "reagent",
                            }
                        ],
                        "operator": "user1",
                        "unit": "",
                        "value": "Cloudy",
                    },
                    {
                        "code": "COLQUAL",
                        "date": date,
                        "flags": ["A"],
                        "instrument": "S001802",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 11, 3, 1, 55, 22),
                                "lot": "K372078",
                                "type": "reagent",
                            }
                        ],
                        "operator": "user1",
                        "unit": "",
                        "value": "Red",
                    },
                    {
                        "code": "GLUQUAL",
                        "date": date,
                        "flags": [],
                        "instrument": "S001802",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 11, 3, 1, 55, 22),
                                "lot": "K372078",
                                "type": "reagent",
                            }
                        ],
                        "operator": "user1",
                        "unit": "",
                        "value": "Negative",
                    },
                    {
                        "code": "KETQUAL",
                        "date": date,
                        "flags": [],
                        "instrument": "S001802",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2017, 11, 3, 1, 55, 22),
                                "lot": "K372078",
                                "type": "reagent",
                            }
                        ],
                        "operator": "user1",
                        "unit": "",
                        "value": "Negative",
                    },
                ],
                "sample_id": "0064",
                "specimen": "UR",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2017, 10, 27, 9, 10, 30)
    worklist = self.decode(
        """
    MSH|^~\\&|Atellica UAS800^1||||20171027094314||OUL^R22^OUL_R22|20171027094314617|P|2.5|||NE|AL||ASCII
    SFT|Atellica UAS 800|4.0.90.5575|Atellica UAS 800|4.0.90.5575
    PID|1||1||Name in user sw
    SPM|1|0064||UR^Urine^HL70487|||||||P^Patient^HL70369
    SAC||||||||||1|5
    OBR|1|||UrineSedimentResult
    ORC|RE||0064||CM|||||test|admini
    NTE|1||Sediment comment in user sw
    NTE|2||Review|
    NTE|3||Low Level|RE
    NTE|4||Dilution factor2.5|RE
    OBX|1|ST|798-9^RBC^LN|1|132|p/ul||A|||F|||||||H100016|20171027091030
    OBX|2|ST|51487-7^WBC^LN|2|267.3|p/ul||A|||F|||||||H100016|20171027091030
    OBX|3|ST|53317-4^.WBCc^LN|3|-|||N|||F|||||||H100016|20171027091030
    OBX|4|ST|53334-9^CRY^LN|4|+++|||A|||F|||||||H100016|20171027091030
    OBX|5|ST|50231-0^HYA^LN|5|++++|||A|||F|||||||H100016|20171027091030
    OBX|6|ST|72224-9^PAT^LN|6|+|||A|||F|||||||H100016|20171027091030
    OBX|7|ST|50225-2^NEC^LN|7|-|||N|||F|||||||H100016|20171027091030
    OBX|8|ST|53318-2^EPI^LN|8|++++|||A|||F|||||||H100016|20171027091030
    OBX|9|ST|72223-1^YEA^LN|9|-|||N|||F|||||||H100016|20171027091030
    OBX|10|ST|50221-1^BAC^LN|10|++|||A|||F|||||||H100016|20171027091030
    OBX|11|ST|^.BACr^LN|11|+|||A|||F|||||||H100016|20171027091030
    OBX|12|ST|^.BACc^LN|12|++|||A|||F|||||||H100016|20171027091030
    OBX|13|ST|53321-6^MUC^LN|13|+|||A|||F|||||||H100016|20171027091030
    OBX|14|ST|33232-0^SPRM^LN|14|+|||A|||F|||||||H100016|20171027091030
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "RBC",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "p/ul",
                        "value": "132",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": "WBC",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "p/ul",
                        "value": "267.3",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": ".WBCcQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "-",
                        "flags": [],
                        "lots": [],
                    },
                    {
                        "code": "NECQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "-",
                        "flags": [],
                        "lots": [],
                    },
                    {
                        "code": "EPIQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "++++",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": "YEAQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "-",
                        "flags": [],
                        "lots": [],
                    },
                    {
                        "code": "BACQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "++",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": ".BACrQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "+",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": ".BACcQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "++",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": "MUCQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "+",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {
                        "code": "SPRMQUAL",
                        "date": date,
                        "instrument": "H100016",
                        "operator": "",
                        "unit": "",
                        "value": "+",
                        "flags": ["A"],
                        "lots": [],
                    },
                    {"code": "CRISTAUX", "value": "CRY+++"},
                    {"code": "CYLINDRES", "value": "HYA++++;PAT+"},
                ],
                "sample_id": "0064",
                "specimen": "UR",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
