# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "b221"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||Laboratoire^Roche^OMNI S^V7.09^1^2639^192.168.1.4||||||PQ|P|1394-97|20140306093219
    Q|1|^SAMPLE1||||||||||D
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "SAMPLE1"}],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&||||||||||P|1394-97|20130715141912
        P|1||036666666||NYME^ANO|NJF||F
        O|1|^123456||^^^A\\^^^B||20130513114242|||||A|||||prescriber|||||||||O
        L|1|N"""
    assert result == expected


def test_encoder_order_without_prescriber(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["physicians"] = []
    result = self.order(worklist=worklist)
    expected = """H|\\^&||||||||||P|1394-97|20130715141912
        P|1||036666666||NYME^ANO|NJF||F
        O|1|^123456||^^^A\\^^^B||20130513114242|||||A||||||||||||||O
        L|1|N"""
    assert result == expected


def test_result(self):
    date = datetime.datetime(2014, 3, 6, 9, 32, 16)
    result = self.decode(
        """
    H|\\^&|||Laboratoire^Roche^OMNI S^V7.09^1^2639^192.168.1.4||||||M|P|1394-97|20140306093535
    P|1|||||||U||||||||0.0^cm|0.0^kg
    O|1|83014351|^22558^^^^Syringe||||||||||||Blood^Arterial^A. femoralis l.
    R|1|^^^pH^^^M^1|7.417||7.350^7.450^reference\\7.200^7.600^critical|N||F||||20140306093216
    R|2|^^^PO2^^^M^3|199.7|mmHg|80.0^100.0^reference\\60.0^800.0^critical|H||F
    R|3|^^^PCO2^^^M^4|40.0|mmHg|35.0^45.0^reference\\20.0^60.0^critical|N||F
    R|4|^^^tHb^^^M^10|8.3|g/dL|11.5^17.4^reference\\8.0^23.0^critical|L||F
    R|5|^^^SO2^^^M^11|99.7|%|75.0^99.0^reference\\60.0^100.0^critical|H||F
    R|6|^^^Baro^^^M^31|757.4|mmHg||N||F
    R|7|^^^H+^^^C^50|38.3|nmol/L||N||F
    R|8|^^^cHCO3^^^C^51|25.2|mmol/l||N||F
    R|9|^^^ctCO2(P)^^^C^52|59.2|vol%||N||F
    R|10|^^^BE^^^C^53|0.7|mmol/l||N||F
    R|11|^^^BE(act)^^^C^54|1.2|mmol/l||N||F
    R|12|^^^BEecf^^^C^55|0.7|mmol/l||N||F
    R|13|^^^BB^^^C^56|45.9|mmol/l||N||F
    R|14|^^^SO2(c)^^^C^58|99.7|%||N||F
    R|15|^^^P50^^^C^156|27.4|mmHg||N||F
    R|16|^^^ctO2^^^C^60|12.2|vol%||N||F
    R|17|^^^ctCO2(B)^^^C^61|23.9|mmol/l||N||F
    R|18|^^^pHst^^^C^62|7.417|||N||F
    R|19|^^^cHCO3st^^^C^63|24.9|mmol/l||N||F
    R|20|^^^PAO2^^^C^64|199.7|mmHg||N||F
    R|21|^^^AaDO2^^^C^65|0.0|mmHg||N||F
    R|22|^^^a/AO2^^^C^66|100.0|%||N||F
    R|23|^^^avDO2^^^C^67||vol%||A||F
    R|24|^^^RI^^^C^68|0|%||N||F
    R|25|^^^Qs/Qt^^^C^69|0.76|%||N||F
    R|26|^^^OER^^^C^83||%||A||F
    R|27|^^^pht^^^C^72|7.417|||N||F
    R|28|^^^cHt^^^C^73|38.3|nmol/L||N||F
    R|29|^^^PCO2t^^^C^74|40.0|mmHg||N||F
    R|30|^^^PO2t^^^C^75|199.7|mmHg||N||F
    R|31|^^^PAO2t^^^C^76|199.7|mmHg||N||F
    R|32|^^^AaDO2t^^^C^77|0.0|mmHg||N||F
    R|33|^^^a/AO2t^^^C^78|100.0|%||N||F
    R|34|^^^RIt^^^C^79|0|%||N||F
    R|35|^^^Hct(c)^^^C^80|25.0|%||N||F
    R|36|^^^MCHC^^^C^81||g/dL||N||F
    R|37|^^^Qt^^^C^86||vol%||A||F
    R|38|^^^PFIndex^^^C^88|951.1|mmHg||N||F
    R|39|^^^ALLEN test^^^I^152|Acceptable|||N||F
    R|40|^^^Pat.Temp^^^I^155|37.0|C||N||F
    R|41|^^^R^^^I^157|0.84|||N||F
    R|42|^^^FIO2^^^I^158|0.21|||N||F
    R|43|^^^tHb(e)^^^I^171|15.0|g/dL||N||F
    R|44|^^^Hb Factor^^^I^172|3.0|||N||F
    R|45|^^^24h Urine^^^I^159||ml||N||F
    R|46|^^^Vent Mode^^^I^160||||N||F
    R|47|^^^VT^^^I^161||l||N||F
    R|48|^^^MV^^^I^162||l||N||F
    R|49|^^^PIP^^^I^163||cmH2O||N||F
    R|50|^^^Ti^^^I^164||s||N||
    R|51|^^^Te^^^I^165||s||N||F
    R|52|^^^SRATE^^^I^166||||N||F
    R|53|^^^ARATE^^^I^167||||N||F
    R|54|^^^PEEP^^^I^168||cmH2O||N||F
    R|55|^^^MAP^^^I^169||cmH2O||N||F
    R|56|^^^Flow^^^I^170||l/min||N||F
    R|57|^^^Age (A/F)^^^I^110|> 1 year|||N||F
    R|58|^^^Date drawn^^^I^145||||N||F
    R|59|^^^Time drawn^^^I^146||||N||F
    R|60|^^^Date changed^^^I^138|20140306|||N||F
    R|61|^^^Time changed^^^I^137|093218|||N||F
    R|62|^^^Department^^^I^174||||N||
    R|63|^^^Accepted by^^^I^144||||N||F
    R|64|^^^Billing code^^^I^173||||N||F
    R|65|^^^Remark^^^I^140||||N||F
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "83014351",
                "results": [
                    {"flags": ["N"], "date": date, "value": "7.417", "code": "pH"},
                    {"flags": ["H"], "date": date, "value": "199.7", "code": "PO2"},
                    {"flags": ["N"], "date": date, "value": "40.0", "code": "PCO2"},
                    {"flags": ["L"], "date": date, "value": "8.3", "code": "tHb"},
                    {"flags": ["H"], "date": date, "value": "99.7", "code": "SO2"},
                    {"flags": ["N"], "date": date, "value": "757.4", "code": "Baro"},
                    {"flags": ["N"], "date": date, "value": "38.3", "code": "H+"},
                    {"flags": ["N"], "date": date, "value": "25.2", "code": "cHCO3"},
                    {"flags": ["N"], "date": date, "value": "59.2", "code": "ctCO2(P)"},
                    {"flags": ["N"], "date": date, "value": "0.7", "code": "BE"},
                    {"flags": ["N"], "date": date, "value": "1.2", "code": "BE(act)"},
                    {"flags": ["N"], "date": date, "value": "0.7", "code": "BEecf"},
                    {"flags": ["N"], "date": date, "value": "45.9", "code": "BB"},
                    {"flags": ["N"], "date": date, "value": "99.7", "code": "SO2(c)"},
                    {"flags": ["N"], "date": date, "value": "27.4", "code": "P50"},
                    {"flags": ["N"], "date": date, "value": "12.2", "code": "ctO2"},
                    {"flags": ["N"], "date": date, "value": "23.9", "code": "ctCO2(B)"},
                    {"flags": ["N"], "date": date, "value": "7.417", "code": "pHst"},
                    {"flags": ["N"], "date": date, "value": "24.9", "code": "cHCO3st"},
                    {"flags": ["N"], "date": date, "value": "199.7", "code": "PAO2"},
                    {"flags": ["N"], "date": date, "value": "0.0", "code": "AaDO2"},
                    {"flags": ["N"], "date": date, "value": "100.0", "code": "a/AO2"},
                    {"flags": ["A"], "date": date, "value": "", "code": "avDO2"},
                    {"flags": ["N"], "date": date, "value": "0", "code": "RI"},
                    {"flags": ["N"], "date": date, "value": "0.76", "code": "Qs/Qt"},
                    {"flags": ["A"], "date": date, "value": "", "code": "OER"},
                    {"flags": ["N"], "date": date, "value": "7.417", "code": "pht"},
                    {"flags": ["N"], "date": date, "value": "38.3", "code": "cHt"},
                    {"flags": ["N"], "date": date, "value": "40.0", "code": "PCO2t"},
                    {"flags": ["N"], "date": date, "value": "199.7", "code": "PO2t"},
                    {"flags": ["N"], "date": date, "value": "199.7", "code": "PAO2t"},
                    {"flags": ["N"], "date": date, "value": "0.0", "code": "AaDO2t"},
                    {"flags": ["N"], "date": date, "value": "100.0", "code": "a/AO2t"},
                    {"flags": ["N"], "date": date, "value": "0", "code": "RIt"},
                    {"flags": ["N"], "date": date, "value": "25.0", "code": "Hct(c)"},
                    {"flags": ["N"], "date": date, "value": "", "code": "MCHC"},
                    {"flags": ["A"], "date": date, "value": "", "code": "Qt"},
                    {"flags": ["N"], "date": date, "value": "951.1", "code": "PFIndex"},
                    {
                        "flags": ["N"],
                        "date": date,
                        "value": "Acceptable",
                        "code": "ALLEN test",
                    },
                    {"flags": ["N"], "date": date, "value": "37.0", "code": "Pat.Temp"},
                    {"flags": ["N"], "date": date, "value": "0.84", "code": "R"},
                    {"flags": ["N"], "date": date, "value": "0.21", "code": "FIO2"},
                    {"flags": ["N"], "date": date, "value": "15.0", "code": "tHb(e)"},
                    {"flags": ["N"], "date": date, "value": "3.0", "code": "Hb Factor"},
                    {"flags": ["N"], "date": date, "value": "", "code": "24h Urine"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Vent Mode"},
                    {"flags": ["N"], "date": date, "value": "", "code": "VT"},
                    {"flags": ["N"], "date": date, "value": "", "code": "MV"},
                    {"flags": ["N"], "date": date, "value": "", "code": "PIP"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Ti"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Te"},
                    {"flags": ["N"], "date": date, "value": "", "code": "SRATE"},
                    {"flags": ["N"], "date": date, "value": "", "code": "ARATE"},
                    {"flags": ["N"], "date": date, "value": "", "code": "PEEP"},
                    {"flags": ["N"], "date": date, "value": "", "code": "MAP"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Flow"},
                    {
                        "flags": ["N"],
                        "date": date,
                        "value": "> 1 year",
                        "code": "Age (A/F)",
                    },
                    {"flags": ["N"], "date": date, "value": "", "code": "Date drawn"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Time drawn"},
                    {
                        "flags": ["N"],
                        "date": date,
                        "value": "20140306",
                        "code": "Date changed",
                    },
                    {
                        "flags": ["N"],
                        "date": date,
                        "value": "093218",
                        "code": "Time changed",
                    },
                    {"flags": ["N"], "date": date, "value": "", "code": "Department"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Accepted by"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Billing code"},
                    {"flags": ["N"], "date": date, "value": "", "code": "Remark"},
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    date = datetime.datetime(2014, 3, 6, 6, 0, 36)
    result = self.decode(
        """
    H|\\^&|||Laboratoire^Roche^OMNI S^V7.09^1^2639^192.168.1.4||||||QC|P|1394-97|20140306060447
    P|1
    O|1||||||||||||||AUTO-TROL TS +^1^21431321^aqueous
    C|1|I||G
    R|1|^^^PCO2^^^M^606|64.7|mmHg|61.0^69.0|N||F||SYSTEM||20140306060036
    R|2|^^^pH^^^M^602|7.174||7.140^7.200|N||F
    R|3|^^^PO2^^^M^605|59.6|mmHg|47.0^71.0|N||F
    L|1|F"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "21431321",
                "control_lot": "21431321",
                "results": [
                    {"flags": ["N"], "date": date, "value": "64.7", "code": "PCO2"},
                    {"flags": ["N"], "date": date, "value": "7.174", "code": "pH"},
                    {"flags": ["N"], "date": date, "value": "59.6", "code": "PO2"},
                ],
            }
        ],
    }
    assert result == expected


def test_segment_m(self):
    result = self.decode(
        """
    H|\\^&|||Laboratoire^Roche^OMNI S^V7.09^1^2639^192.168.1.4||||||SR^REAL|P|1394-97|20140402151855
    M|1|SR^RO^OS^1|308^PO2^1P Pot|395.20|mV|330.00^250.00^550.00|N^0|SYSTEM|20140402151855
    M|2|SR^RO^OS^1|312^PCO2^1P Pot|-1017.58|mV|-1000.00^-1300.00^2000.00|N^0
    M|3|SR^RO^OS^1|316^pH^1P Pot|536.39|mV|500.00^-2100.00^1600.00|N^0
    M|4|SR^RO^OS^1|31^Baro|739.0|mmHg| 0.0^450.0^800.0|N^0
    M|5|SR^RO^OS^1|374^Cal type|1P cal|||N^0
    L|1|N
    """
    )
    expected = {"context": "result", "samples": []}
    assert result == expected
