# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "panther"


def test_encode_order(self):
    self.configure(panther_sender_id="MCA", panther_receiver_id="PANTHER")
    message = self.order()
    expected = """
    H|\\^&|||MCA|||||PANTHER||P|1|20130715141912
    P|1|036666666|||NYME^ANO||20110513|F|||||ATTENDING
    O|1|123456||^^^A^^^1:10|S|20130603110342|||||N||||Serum||||||||||O
    O|2|123456||^^^B|S|20130603110342|||||A||||Serum||||||||||O
    O|3|123456||^^^C|S|20130603110342|||||C||||Serum||||||||||O
    L|1|N"""
    assert message == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
        H|\\^&|||LISHost|||||Panther||P|1
        Q|1|^12345||ALL||||||||O
        L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||LISHost|||||Panther||P|1|20130715141912
    P|1
    O|1|12345|||||||||||||||||||||||Y
    L|1|I"""
    assert result == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||LISHost|||||Panther||P|1
    Q|1|^ALL||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "ALL", "type": "patient"}],
    }
    assert worklist == expected


def test_decode_result_not_processed(self):
    worklist = self.decode(
        """
    H|\\^&|||LISHost|||||Panther||P|1
    M|1|5.3.2.9|6.0.0.1|John Smith^Hologic Inc.^SW Department^Building 2^10210 Genetic Center Dr.^San Diego, CA^92121
    P|1|PatID01|||Meier^Anna||1974100123|F|||||MARTINEZ
    O|1|SAMPLE01|^|^^^CT/GC^^^1|||||||||||||||||||||X
    R|1|^^^CT/GC^^^||||||X
    C|1|I|RJ_notProcessed^|I
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CT/GC",
                        "comments": [],
                        "date": None,
                        "dilution": "",
                        "flags": ["RJ_notProcessed"],
                        "unit": "",
                        "value": "",
                    }
                ],
                "sample_id": "SAMPLE01",
                "specimen": "",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_patient(self):
    date = datetime.datetime(2013, 5, 18, 0, 17, 47)
    worklist = self.decode(
        """
    H|\\^&|||Panther|||||LISHost||P|1|
    P|1|PatID01|||Meier^Anna^||1974100123|F|||||MARTINEZ|||^|^|||||||||||||||^|^|
    O|1|SAMPLE01|f72423fb-857d-4a97-9247-a26803c45070^19271|^^^GT HPV^HPV 16 18/45^^1||20130517204617|||||||||||||||||||F
    M|1|45316.667^358857.000^1688731.333^6165206.333^1584.667^3939.333|113202.000^82081.333^56221.333^0.000^3155736.333^120187.333|0^0^104.642506533195^98.5671926511015^87.6482741453833^108.145588770874|1000^48074^1.16319513576374^1017599
    R|1|^^^GT HPV^IC/HPV 16 RLU^^1|2319091|||||F\\R||||20130518001747
    R|2|^^^GT HPV^HPV 16 S/CO^^1|3.67|||||F\\R||||20130518001747
    R|3|^^^GT HPV^HPV 16 Result^^1|POSITIVE|||||F\\R||||20130518001747
    R|4|^^^GT HPV^HPV 18/45 RLU^^1|0|||||F\\R||||20130518001747
    R|5|^^^GT HPV^HPV 18/45 S/CO^^1|0.00|||||F\\R||||20130518001747
    R|6|^^^GT HPV^HPV 18/45 Result^^1|Negative|||||F\\R||||20130518001747
    R|7|^^^GT HPV^ICInterpretation^^1|Valid|||||F\\R||||20130518001747
    R|8|^^^GT HPV^HPV 16 Cutoff^^1||||||F\\R||||20130518001747
    R|9|^^^GT HPV^HPV 18/45 Cutoff^^1||||||F\\R||||20130518001747
    R|10|^^^GT HPV^IC Cutoff^^1||||||F\\R||||20130518001747
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "SAMPLE01",
                "specimen": "",
                "results": [
                    {
                        "code": "GT HPVIC/HPV 16 RLU",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "2319091",
                    },
                    {
                        "code": "GT HPVHPV 16 S/CO",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "3.67",
                    },
                    {
                        "code": "GT HPVHPV 16 Result",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "POSITIVE",
                    },
                    {
                        "code": "GT HPVHPV 18/45 RLU",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "0",
                    },
                    {
                        "code": "GT HPVHPV 18/45 S/CO",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "0.00",
                    },
                    {
                        "code": "GT HPVHPV 18/45 Result",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Negative",
                    },
                    {
                        "code": "GT HPVICInterpretation",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Valid",
                    },
                    {
                        "code": "GT HPVHPV 16 Cutoff",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "",
                    },
                    {
                        "code": "GT HPVHPV 18/45 Cutoff",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "",
                    },
                    {
                        "code": "GT HPVIC Cutoff",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_patient_2(self):
    date = datetime.datetime(2011, 1, 26, 11, 39, 24)
    worklist = self.decode(
        """
    H|\\^&|||Panther|||||LISHost||P|1|
    P|1|PatID01|||Meier^Anna^||1974100123|F|||||MARTINEZ|||^|^|||||||||||||||^|^|
    O|1|SAMPLE01|925f7749-d99d-4ae5-b093-0757230617f1^17355|^^^dHCV^HCV^^1||20110126073327|||||||||||||||||||F
    R|1|^^^dHCV^ICRLU^^1|209058|||||F\\R||||20110126113924
    R|2|^^^dHCV^ICInterpretation^^1|Valid|||||F\\R||||20110126113924
    R|3|^^^dHCV^AnalyteRLU^^1|4272|||||F\\R||||20110126113924
    R|4|^^^dHCV^AnalyteSCO^^1|0.09|||||F\\R||||20110126113924
    R|5|^^^dHCV^OverallInterpretation^^1|Nonreactive|||||F\\R||||20110126113924
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "SAMPLE01",
                "specimen": "",
                "results": [
                    {
                        "code": "dHCVICRLU",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "209058",
                    },
                    {
                        "code": "dHCVICInterpretation",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Valid",
                    },
                    {
                        "code": "dHCVAnalyteRLU",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "4272",
                    },
                    {
                        "code": "dHCVAnalyteSCO",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "0.09",
                    },
                    {
                        "code": "dHCVOverallInterpretation",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Nonreactive",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_control(self):
    date = datetime.datetime(2011, 1, 26, 11, 39, 24)
    worklist = self.decode(
        """
    H|\\^&|||Panther|||||LISHost||P|1|
    P|1|PatID01|||Meier^Anna^||1974100123|F|||||MARTINEZ|||^|^|||||||||||||||^|^|
    O|1|011228022115092238746|925f7749-d99d-4ae5-b093-0757230617f1^17355|^^^dHBV^HBV^^1||20110126073327|||||Q||||||||||||||F
    R|1|^^^dHBV^AnalyteRLU^^1|4075|||||F\\R||||20110126113924
    R|2|^^^dHBV^AnalyteSCO^^1|0.06|||||F\\R||||20110126113924
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "280221",
                "control_lot": "280221",
                "specimen": "",
                "results": [
                    {
                        "code": "dHBVAnalyteRLU",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "4075",
                    },
                    {
                        "code": "dHBVAnalyteSCO",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "0.06",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_calibrator_as_qc(self):
    worklist = self.decode(
        """
    H|\\^&|||PANTHER|||||HOST||P|1|
    P|1||||^^|||||||||||^|^|||||||||||||||^|^|
    O|1|142428112415092230958|b41769b7-8520-4672-9227-42dfb5325a53^324232|^^^HPV^HPV^^1|R|20210621085903|||||||||Calibrator||||||||||F
    R|1|^^^HPV^ICRLU^^1|303300|||||F\\R|||||
    R|2|^^^HPV^ICInterpretation^^1|Valid|||||F\\R|||||
    R|3|^^^HPV^AnalyteRLU^^1|1146205|||||F\\R|||||
    R|4|^^^HPV^AnalyteSCO^^1|10,66|||||F\\R|||||
    R|5|^^^HPV^OverallInterpretation^^1|Valid|||||F\\R|||||
    R|6|^^^HPV^Analyte Cutoff^^1||||||F\\R|||||
    R|7|^^^HPV^IC Cutoff^^1||||||F\\R|||||
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "281124",
                "control_lot": "281124",
                "specimen": "Calibrator",
                "results": [
                    {
                        "code": "HPVICRLU",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "303300",
                    },
                    {
                        "code": "HPVICInterpretation",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Valid",
                    },
                    {
                        "code": "HPVAnalyteRLU",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "1146205",
                    },
                    {
                        "code": "HPVAnalyteSCO",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "10,66",
                    },
                    {
                        "code": "HPVOverallInterpretation",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Valid",
                    },
                    {
                        "code": "HPVAnalyte Cutoff",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "",
                    },
                    {
                        "code": "HPVIC Cutoff",
                        "comments": [],
                        "date": None,
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_comments(self):
    worklist = self.decode(
        """
    H|\\^&|||Panther|||||Host||P|1|
    P|4|510136|||^^||19930926|F||||||||^|^|||||||||||||||^|^|
    O|1|456456456456|4145297a-8df4-4329-a70b-7805364f9784^2930|^^^CT/GC^GC CT^^1|R|20230120132459|||||||||||||||||||F
    R|1|^^^CT/GC^TotalRLU^^1|0|||||F||||20230120165550|
    C|1|I|VVFS - LLS measurement of TCR/Sample in MTU tube falls outside the expected volume^|I
    C|2|I|^RDFS - Sample dispense verification failed.\\^CLT - Clot detected|I
    C|3|I|^p - Assay Processing Error.|I
    R|2|^^^CT/GC^CTResult^^1|Invalid|||||F||||20230120165550|
    C|1|I|^VVFS - LLS measurement of TCR/Sample in MTU tube falls outside the expected volume|I
    C|2|I|^RDFS - Sample dispense verification failed.|I
    C|3|I|^p - Assay Processing Error.|I
    R|3|^^^CT/GC^GCResult^^1|Invalid|||||F||||20230120165550|
    C|1|I|^VVFS - LLS measurement of TCR/Sample in MTU tube falls outside the expected volume|I
    C|2|I|^RDFS - Sample dispense verification failed.|I
    C|3|I|^p - Assay Processing Error.|I
    L|1|N

    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CT/GCTotalRLU",
                        "comments": [
                            "RDFS - Sample dispense verification " "failed.",
                            "CLT - Clot detected",
                            "p - Assay Processing Error.",
                        ],
                        "date": datetime.datetime(2023, 1, 20, 16, 55, 50),
                        "dilution": "1",
                        "flags": [
                            "VVFS - LLS measurement of TCR/Sample in "
                            "MTU tube falls outside the expected "
                            "volume"
                        ],
                        "unit": "",
                        "value": "0",
                    },
                    {
                        "code": "CT/GCCTResult",
                        "comments": [
                            "VVFS - LLS measurement of TCR/Sample "
                            "in MTU tube falls outside the "
                            "expected volume",
                            "RDFS - Sample dispense verification " "failed.",
                            "p - Assay Processing Error.",
                        ],
                        "date": datetime.datetime(2023, 1, 20, 16, 55, 50),
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Invalid",
                    },
                    {
                        "code": "CT/GCGCResult",
                        "comments": [
                            "VVFS - LLS measurement of TCR/Sample "
                            "in MTU tube falls outside the "
                            "expected volume",
                            "RDFS - Sample dispense verification " "failed.",
                            "p - Assay Processing Error.",
                        ],
                        "date": datetime.datetime(2023, 1, 20, 16, 55, 50),
                        "dilution": "1",
                        "flags": [],
                        "unit": "",
                        "value": "Invalid",
                    },
                ],
                "sample_id": "456456456456",
                "specimen": "",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
