# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.tests import BaseTest

MESSAGE = """MSH|^~\\&|AUTOPLAK|Microbiology|LIS01||20130305092544||OUL^R22^OUL_R22|MSG00001|P|2.5
SPM|1|A239852|1234567890|UR^URINE|||||||P|||||20130304223102
SAC|||1234567890^AUTOPLAK|||||R^COMPLETED|RACK||7|DRAWER|0|4||||||||||||||||USR1
OBR|1|98765_1||URIGLEB|||||||||||||||||||||F
ORC|SC|98765_1||98765|||||20130112131517
OBX|1||URIGLEB||||||||F"""


class TestProtocolAutoplakMLLP(BaseTest):
    PROTOCOL_CLASS = "autoplak"

    def test_send_recv(self):
        self.assertSendRecvMessageEqual(
            MESSAGE,
            """send <vt>MSH|^~\\&|AUTOPLAK|Microbiology|LIS01||20130305092544||OUL^R22^OUL_R22|MSG00001|P|2.5<cr>SPM|1|A239852|1234567890|UR^URINE|||||||P|||||20130304223102<cr>SAC|||1234567890^AUTOPLAK|||||R^COMPLETED|RACK||7|DRAWER|0|4||||||||||||||||USR1<cr>OBR|1|98765_1||URIGLEB|||||||||||||||||||||F<cr>ORC|SC|98765_1||98765|||||20130112131517<cr>OBX|1||URIGLEB||||||||F<cr><fs><cr>
            recv <vt>MSH|^~\\&|LIS01||AUTOPLAK||20130305092544||ACK^R22^ACK|MSG00002|P|2.5|<cr>MSA|AA|MSG00001<fs><cr>""",
        )

    def test_ack(self):
        message = self.P.get_ack_message(
            MESSAGE.encode(),
            date=datetime.datetime(2014, 1, 1, 12, 42, 24),
            control_id=b"CONTROLID",
        )
        expected = b"MSH|^~\\&|LIS01||AUTOPLAK||20140101124224||ACK^R22^ACK|CONTROLID|P|2.5\rMSA|AA|MSG00001|\r"
        assert message == expected
