# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "sofia"


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||Sofia^12345678|||||||P|1.0.2|20171025165023
    P|1|PID1234|||||||||||||||||||||||SITENAME
    O|1|SAM1234||Flu A+B||||||JSmith|||||P
    C|1||Read-Now Mode
    R|1|^^^Flu A|negative|||H||F||||20171025114534
    R|2|^^^Flu B|negative|||||F||||20171025114534
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "SAM1234",
                "results": [
                    {
                        "code": "Flu A",
                        "comments": [],
                        "date": datetime.datetime(2017, 10, 25, 11, 45, 34),
                        "flags": ["H"],
                        "value": "negative",
                        "operator": "JSmith",
                        "instrument": "12345678",
                    },
                    {
                        "code": "Flu B",
                        "comments": [],
                        "date": datetime.datetime(2017, 10, 25, 11, 45, 34),
                        "flags": [""],
                        "value": "negative",
                        "operator": "JSmith",
                        "instrument": "12345678",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||Sofia^12345678|||||||P|1.0.2|20171025165023
    P|1|CASSER12|||||||||||||||||||||||SITENAME
    O|1|KITLOT12||Flu A+B||||||987654|||||Q
    C|1||Read-Now Mode
    R|1|^^^POS|passed|||||F||||20171025084534
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "KITLOT12",
                "control_lot": "KITLOT12",
                "results": [
                    {
                        "code": "POS",
                        "comments": [],
                        "date": datetime.datetime(2017, 10, 25, 8, 45, 34),
                        "flags": [""],
                        "value": "passed",
                        "operator": "987654",
                        "instrument": "12345678",
                    }
                ],
            }
        ],
    }
    assert worklist == expected
