# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "streamlab"


def test_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].orders.add(code="1234^2202", action="new", priority="A")
    result = self.order(worklist=worklist)
    expected = """
    H|\\^&|||NEO|||||STREAMLAB||P|DB 100102|20130715141912
    P|1|036666666|||NYME^ANO|NJF|20110513|F|||||PRESCRIBER
    O|1|123456||^^^A\\^^^B\\^^^1234^2202|S|20130603110342|20130513114242||||A||||Serum|||1|3||||||O
    O|2|123456||^^^C|S|20130603110342|20130513114242||||C||||Serum|||1|1||||||O
    L|1|N"""
    assert result == expected


def test_encode_order_without_prescriber(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["physicians"] = []
    result = self.order(worklist=worklist)
    expected = """
    H|\\^&|||NEO|||||STREAMLAB||P|DB 100102|20130715141912
    P|1|036666666|||NYME^ANO|NJF|20110513|F
    O|1|123456||^^^A\\^^^B|S|20130603110342|20130513114242||||A||||Serum|||1|2||||||O
    O|2|123456||^^^C|S|20130603110342|20130513114242||||C||||Serum|||1|1||||||O
    L|1|N"""
    assert result == expected


def test_rack(self):
    result = self.decode(
        """
    H|\\^&|3829||DBW|||||SIEMENS|||DB 100101
    C|1|I|S002^05133524^RPO-1-A03|S
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [{"sample_id": "05133524", "type": "patient", "rack": "RPO-1-A03"}],
    }
    assert result == expected


def test_resu(self):
    result = self.decode(
        """
        H|\\^&|177||W054|||||STREAMLAB|||DB 100101|
        P|1||||NEOV6^X^^||19470101|F|||||TURNER Luc^^^^||||||||||\\||^|||||||||
        O|1|025116||^^^1607^0401^\\^^^2035^0401^\\^^^2017^0401^\\^^^0312^0401^\\^^^1903^0401^\\^^^0835^0401^HIL Determination|S||||||A||||S^|^^^^||1|6|||20150225131458|||P|||||
        C|1|I|S001^R109B07|S
        R|1|^^^1607^0401^|4.4|mmol/L||||S||||20150225140027|RXL1
        C|1|I|E121^Never Calibrated
        R|2|^^^2035^0401^|30.5|mmol/L||||S||||20150225140027|RXL1
        R|6|^^^0835^0401^HIL Determination|111|Ind||||S||||20150225140027|RXL1
        L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "025116",
                "type": "patient",
                "rack": "R109B07",
                "specimen": "S",
                "results": [
                    {
                        "instrument": "RXL1",
                        "date": datetime.datetime(2015, 2, 25, 14, 00, 27),
                        "value": "4.4",
                        "code": "1607",
                        "flags": ["E121"],
                    },
                    {
                        "instrument": "RXL1",
                        "date": datetime.datetime(2015, 2, 25, 14, 00, 27),
                        "value": "30.5",
                        "code": "2035",
                        "flags": [],
                    },
                    {
                        "instrument": "RXL1",
                        "date": datetime.datetime(2015, 2, 25, 14, 00, 27),
                        "value": "111",
                        "code": "0835",
                        "flags": [],
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|13||W054|||||SIEMENS|||DB 100101|
    P|1||QC CAL LDI - SerumCQ||^^^||||||||^^^^||||||||||\\||^|||||||||
    O|1|RXL1BA51016||^^^4018^0401^LDI|A||||||Q||||O^QC2|^^^^||1|1|||20150227091848|||P|||||
    R|1|^^^4018^0401^LDI|161|U/L||||S||||20150227091848|RXL2
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "RXL1BA51016",
                "control_lot": "RXL1BA51016",
                "type": "control",
                "specimen": "O",
                "results": [
                    {
                        "instrument": "RXL2",
                        "date": datetime.datetime(2015, 2, 27, 9, 18, 48),
                        "value": "161",
                        "code": "4018",
                        "flags": [],
                    }
                ],
            }
        ],
    }
    assert result == expected
