# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import os

parser_class = "thermofisher"


def get_data_file(filename, decode=False, encoding="utf-8"):
    filepath = os.path.join(os.path.dirname(__file__), filename)
    mode = "rb"
    with open(filepath, mode) as f:
        data = f.read().decode("utf-8")
    return data


def test_decode_result(self):
    self.configure(analyseur_traites="SVT005")
    data = get_data_file("test_thermofisher.csv")
    worklist = self.decode(data)
    date_resu = datetime.datetime(2015, 4, 10, 9, 56, 43)
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "1-A1",
                "results": [
                    {
                        "code": "KAZ_Task",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "NTC",
                    },
                    {
                        "code": "KAZ_Reporter",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "FAM",
                    },
                    {
                        "code": "KAZ_Quencher",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "NFQ-MGB",
                    },
                    {
                        "code": "KAZ_AmpStatus",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "No Amp",
                    },
                    {
                        "code": "KAZ_AmpScore",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "0.0",
                    },
                    {
                        "code": "KAZ_Cq",
                        "instrument": "SVT005",
                        "value": "Undetermined",
                        "date": date_resu,
                    },
                    {
                        "code": "KAZ_CqConfidence",
                        "instrument": "SVT005",
                        "value": "0.0",
                        "date": date_resu,
                    },
                    {
                        "code": "KAZ_AutoThreshold",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "true",
                    },
                    {
                        "code": "KAZ_Threshold",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "0.37391034287491404",
                    },
                    {
                        "code": "KAZ_AutoBaseline",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "true",
                    },
                    {
                        "code": "KAZ_BaselineStart",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "3",
                    },
                    {
                        "code": "KAZ_BaselineEnd",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "39",
                    },
                ],
                "sample_id": "NTC",
                "type": "patient",
            },
            {
                "rack": "3-A3",
                "results": [
                    {
                        "code": "KAZ_Task",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "STANDARD",
                    },
                    {
                        "code": "KAZ_Reporter",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "FAM",
                    },
                    {
                        "code": "KAZ_Quencher",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "NFQ-MGB",
                    },
                    {
                        "code": "KAZ_AmpStatus",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "Inconclusive",
                    },
                    {
                        "code": "KAZ_AmpScore",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "1.2844791349400009",
                    },
                    {
                        "code": "KAZ_Cq",
                        "instrument": "SVT005",
                        "value": "37.659820580900515",
                        "date": date_resu,
                    },
                    {
                        "code": "KAZ_CqConfidence",
                        "instrument": "SVT005",
                        "value": "0.9281922826265063",
                        "date": date_resu,
                    },
                    {
                        "code": "KAZ_CqMean",
                        "instrument": "SVT005",
                        "value": "36.870508255399955",
                        "date": date_resu,
                    },
                    {
                        "code": "KAZ_CqSD",
                        "instrument": "SVT005",
                        "value": "0.5581384803294157",
                        "date": date_resu,
                    },
                    {
                        "code": "KAZ_AutoThreshold",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "true",
                    },
                    {
                        "code": "KAZ_Threshold",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "0.37391034287491404",
                    },
                    {
                        "code": "KAZ_AutoBaseline",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "true",
                    },
                    {
                        "code": "KAZ_BaselineStart",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "3",
                    },
                    {
                        "code": "KAZ_BaselineEnd",
                        "date": date_resu,
                        "instrument": "SVT005",
                        "value": "32",
                    },
                ],
                "sample_id": "1E1",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected
