# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "vista"


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|1||VISTA 1|||||HOST|||DB 100102|20140603120334|
    Q|1|^01445637||||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "01445637"}],
    }
    assert worklist == expected


def test_encode_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            H|\\^&|1||VISTA 1|||||HOST|||DB 100102|20140603120334|
            Q|1|^01445637||||||||||O
            L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||MCA|||||VISTA||P|DB 100102|20130715141912
    Q|1|^01445637||||||||||X
    L|1|I"""
    assert message == expected


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||MCA|||||VISTA||P|DB 100102|20130715141912
    P|1|036666666|||NYME^ANO||20110513121500|F||||||||||||||||||||||||||
    O|1|123456||^^^A\\^^^B|A||20130513114242||||A|||||||1|2||||||O||||||
    L|1|N"""
    assert message == expected


def test_encode_order_priority(self):
    self.configure(send_priority=False)
    message = self.order()
    expected = """
    H|\\^&|||MCA|||||VISTA||P|DB 100102|20130715141912
    P|1|036666666|||NYME^ANO||20110513121500|F||||||||||||||||||||||||||
    O|1|123456||^^^A\\^^^B|||20130513114242||||A|||||||1|2||||||O||||||
    L|1|N"""
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|1||DV330574|||||HOST||P|DB 100102|20130311122303|
    P|1||||ANO^NYME^^||19770929|F|||||^^^^|||||^|||||\\||^||||||||||
    O|1|130311048601||^^^2816^2202^Hemolysis\\^^^2817^2202^Icterus\\^^^2818^2202^Lipemia\\^^^2102^2202^Uric Acid\\^^^0127^2202^Aspartate Aminotransferase\\^^^0146^2202^Alanine Aminotransferase|S||||||N||||P^|^^^^||1|6|||20130311122303|||F|||||
    C|1|I|G001^CY001302-1|G
    R|1|^^^2816^2202^Hemolysis|3|index||||F||ML|20130311121420|20130311121557|VistaDim1
    R|2|^^^2817^2202^Icterus|1|index||||F||ML|20130311121420|20130311121557|VistaDim1
    R|3|^^^2818^2202^Lipemia|1|index||||F||ML|20130311121420|20130311121557|VistaDim1
    R|4|^^^2102^2202^Uric Acid|47.43|mg/L|26.0 - 72.0|||F||ML|20130311121528|20130311122114|VistaDim1
    C|1|I|R001^12265AB|R
    C|2|I|R002^20130921235900|R
    C|3|I|R003^010594|R
    R|5|^^^0127^2202^Aspartate Aminotransferase|16.01|U/L|15.0 - 37.0|||F||ML|20130311121514|20130311122143|VistaDim1
    C|1|I|E111^H Interference|E
    C|2|I|R001^12208BG|R
    C|3|I|R002^20130726235900|R
    C|4|I|R003^004271|R
    R|6|^^^0146^2202^Alanine Aminotransferase|17.63|U/L|12.0 - 78.0|||F||ML|20130311121521|20130311122150|VistaDim1
    C|1|I|R001^12227BB|R
    C|2|I|R002^20130814235900|R
    C|3|I|R003^001120|R
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "130311048601",
                "specimen": "P",
                "rack": "CY001302-1",
                "results": [
                    {
                        "code": "2816",
                        "date": datetime.datetime(2013, 3, 11, 12, 15, 57),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [],
                        "value": "3",
                        "unit": "index",
                        "reference_ranges": "",
                        "operator": "ML",
                    },
                    {
                        "code": "2817",
                        "date": datetime.datetime(2013, 3, 11, 12, 15, 57),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [],
                        "value": "1",
                        "unit": "index",
                        "reference_ranges": "",
                        "operator": "ML",
                    },
                    {
                        "code": "2818",
                        "date": datetime.datetime(2013, 3, 11, 12, 15, 57),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [],
                        "value": "1",
                        "unit": "index",
                        "reference_ranges": "",
                        "operator": "ML",
                    },
                    {
                        "code": "2102",
                        "date": datetime.datetime(2013, 3, 11, 12, 21, 14),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2013, 9, 21, 23, 59),
                                "lot": "12265AB",
                                "type": "reagent",
                            }
                        ],
                        "value": "47.43",
                        "unit": "mg/L",
                        "reference_ranges": "26.0 - 72.0",
                        "operator": "ML",
                    },
                    {
                        "code": "0127",
                        "date": datetime.datetime(2013, 3, 11, 12, 21, 43),
                        "flags": ["E111"],
                        "instrument": "VistaDim1",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2013, 7, 26, 23, 59),
                                "lot": "12208BG",
                                "type": "reagent",
                            }
                        ],
                        "value": "16.01",
                        "unit": "U/L",
                        "reference_ranges": "15.0 - 37.0",
                        "operator": "ML",
                    },
                    {
                        "code": "0146",
                        "date": datetime.datetime(2013, 3, 11, 12, 21, 50),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2013, 8, 14, 23, 59),
                                "lot": "12227BB",
                                "type": "reagent",
                            }
                        ],
                        "value": "17.63",
                        "unit": "U/L",
                        "reference_ranges": "12.0 - 78.0",
                        "operator": "ML",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|1||DV330574|||||HOST||Q|DB 100102|20130311133252|
    P|1||||^^^|||U|||||^^^^|||||^|||||\\||^||||||||||
    O|1|45633VB||^^^0106^2202^Albumin\\^^^0127^2202^Aspartate Aminotransferase\\^^^2101^2202^Blood Urea Nitrogen|R||||||Q||||S^QC3|^^^^||1|26|||20130311133252|||F|||||
    R|1|^^^0106^2202^Albumin|41.56|g/L|38.9 - 44.8|||F||ML|20130311131315|20130311131615|VistaDim1
    C|1|I|R001^12244AB|R
    C|2|I|R002^20130831235900|R
    C|3|I|R003^000666|R
    C|4|I|Q001^45633VB|Q
    C|5|I|Q002^20140228000000|Q
    C|6|I|Q004^ASYD MULTIQUAL|Q
    C|7|I|Q005^A|Q
    R|2|^^^0127^2202^Aspartate Aminotransferase|252.19|U/L|222.0 - 267.0|||F||ML|20130311131116|20130311131745|VistaDim1
    C|1|I|R001^12208BG|R
    C|2|I|R002^20130726235900|R
    C|3|I|R003^004271|R
    C|4|I|Q001^45633VB|Q
    C|5|I|Q002^20140228000000|Q
    C|6|I|Q004^ASYD MULTIQUAL|Q
    C|7|I|Q005^A|Q
    R|3|^^^2101^2202^Blood Urea Nitrogen|1.37|g/L|1.34 - 1.61|||F||ML|20130311131355|20130311131658|VistaDim1
    C|1|I|E550^Correlated|E
    C|2|I|R001^12292BA|R
    C|3|I|R002^20131018235900|R
    C|4|I|R003^001135|R
    C|5|I|Q001^45633VB|Q
    C|6|I|Q002^20140228000000|Q
    C|7|I|Q004^ASYD MULTIQUAL|Q
    C|8|I|Q005^A|Q
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "45633VB",
                "control_lot": "45633VB",
                "control_level": "3",
                "specimen": "S",
                "results": [
                    {
                        "code": "0106",
                        "date": datetime.datetime(2013, 3, 11, 13, 16, 15),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2014, 2, 28, 0, 0),
                                "lot": "45633VB",
                                "name": "ASYD MULTIQUAL",
                                "type": "control",
                            },
                            {
                                "expiration": datetime.datetime(2013, 8, 31, 23, 59),
                                "lot": "12244AB",
                                "type": "reagent",
                            },
                        ],
                        "value": "41.56",
                        "unit": "g/L",
                        "reference_ranges": "38.9 - 44.8",
                        "operator": "ML",
                    },
                    {
                        "code": "0127",
                        "date": datetime.datetime(2013, 3, 11, 13, 17, 45),
                        "flags": [],
                        "instrument": "VistaDim1",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2014, 2, 28, 0, 0),
                                "lot": "45633VB",
                                "name": "ASYD MULTIQUAL",
                                "type": "control",
                            },
                            {
                                "expiration": datetime.datetime(2013, 7, 26, 23, 59),
                                "lot": "12208BG",
                                "type": "reagent",
                            },
                        ],
                        "value": "252.19",
                        "unit": "U/L",
                        "reference_ranges": "222.0 - 267.0",
                        "operator": "ML",
                    },
                    {
                        "code": "2101",
                        "date": datetime.datetime(2013, 3, 11, 13, 16, 58),
                        "flags": ["E550"],
                        "instrument": "VistaDim1",
                        "lots": [
                            {
                                "expiration": datetime.datetime(2014, 2, 28, 0, 0),
                                "lot": "45633VB",
                                "name": "ASYD MULTIQUAL",
                                "type": "control",
                            },
                            {
                                "expiration": datetime.datetime(2013, 10, 18, 23, 59),
                                "lot": "12292BA",
                                "type": "reagent",
                            },
                        ],
                        "value": "1.37",
                        "unit": "g/L",
                        "reference_ranges": "1.34 - 1.61",
                        "operator": "ML",
                    },
                ],
            }
        ],
    }

    assert worklist == expected
